﻿Option Explicit On
Option Strict On
Imports System.Data
Imports System.Data.Linq
Imports System.Math
Imports System.IO
Imports System.Text
Imports System.Linq

Public Class frmLINQ2SQL
    Private dcNwind As NwindLINQDataContext
    Private sbLog As New StringBuilder
    Private swLog As New StringWriter(sbLog)
    Private blnIsLoaded As Boolean
    'Session variables to make adding new objects visible in datagrid views
    Private intOrderID As Integer
    Private strProductIDs As String

    Private Sub frmLINQ2SQL_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Load the controls from the default (Tables)
        blnIsLoaded = False
        dcNwind = New NwindLINQDataContext
        dcNwind.Log = swLog
        LoadFromTables()
        Application.DoEvents()
        blnIsLoaded = True
    End Sub

    Private Sub btnReload_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnReload.Click
        If optLinqQueries.Checked Or optLinqSproc.Checked Then
            LoadFromLinq()
        Else
            LoadFromTables()
        End If
    End Sub

    Private Sub LoadFromTables()
        Try
            Dim timLoad As New Stopwatch
            sbLog.Length = 0
            sbLog.Append("Load from Tables (All Customers)" + vbCrLf)
            'Load as Linq.Table objects (for conventional master/child/grandchild databinding)
            timLoad.Start()
            Me.CustomerBindingSource.DataSource = dcNwind.Customers
            Me.OrdersBindingSource.DataSource = Me.CustomerBindingSource
            Me.OrdersBindingSource.DataMember = "Orders"
            'Terry Adams (MSFT) post in the LINQ Project General Forum: The March CTP contained only limited data binding functionality. 
            'Their [sic] is a SortableBindingList implementation planned for the next Linq to SQL preview.  
            'You can create your own implementation for most situations; but may have some issues with Master-Detail scenarios when binding to an EntitySet<>.  
            Me.Order_DetailsBindingSource.DataSource = Me.OrdersBindingSource
            Me.Order_DetailsBindingSource.DataMember = "Order_Details"
            txtTime.Text = Format(timLoad.ElapsedMilliseconds / 1000, "0.###")
            txtLog.Text = sbLog.ToString
            timLoad = Nothing
        Catch exc As Exception
            MsgBox(exc.Message, MsgBoxStyle.Exclamation, "Exception Loading BindingSources from Tables")
        End Try
    End Sub

    Private Sub LoadFromLinq()
        'Use LINQ queries
        Dim timLoad As New Stopwatch
        sbLog.Length = 0
        sbLog.Append("LoadFromLinq Procedure (US Customers)" + vbCrLf)
        timLoad.Start()
        If optLinqSproc.Checked Then
            'Use the IMultipleResults interface with a single result
            Me.CustomerBindingSource.DataSource = dcNwind.GetCustomersByCountry("USA").GetResult(Of Customer)()
        Else
            'Execute a LINQ query
            Me.CustomerBindingSource.DataSource = _
                From Custs In dcNwind.Customers _
                Where Custs.Country = "USA" _
                Select Custs
        End If
        LoadOrdersGrid()
        LoadOrder_DetailsGrid()
        txtTime.Text = Format(timLoad.ElapsedMilliseconds / 1000, "0.###")
        txtLog.Text = sbLog.ToString
        timLoad = Nothing
    End Sub

    Private Sub LoadOrdersGrid()
        'Reload Orders with LINQ query when CustomerID changes 
        Me.OrdersBindingSource.DataMember = ""
        Me.OrdersBindingSource.DataSource = _
            From Ords In dcNwind.Orders _
            Where Ords.CustomerID = CustomerIDTextBox.Text _
            Order By Ords.OrderID Descending
        Me.OrdersBindingSource.ResetBindings(True)
        Me.OrdersDataGridView.Refresh()
    End Sub

    Private Sub LoadOrder_DetailsGrid()
        'Reload Order_Details with LINQ query when CustomerID changes
        Dim objOrder As Order_ = CType(Me.OrdersBindingSource.Current, Order_)
        Me.Order_DetailsBindingSource.DataMember = ""
        Me.Order_DetailsBindingSource.DataSource = _
            From Dtls In dcNwind.Order_Details _
            Where (Dtls.OrderID = objOrder.OrderID)
        'Where Dtls.OrderID = CInt(OrdersDataGridView.Rows(0).Cells(0).Value)
        Me.OrdersBindingSource.ResetBindings(True)
        Me.OrdersDataGridView.Refresh()
    End Sub

#Region "Synchronize Grids"
    Private Sub CustomerBindingSource_PositionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles CustomerBindingSource.PositionChanged
        If optLinqQueries.Checked Then
            'Reload the Orders grid
            LoadOrdersGrid()
            Me.OrdersBindingSource.MoveFirst()
            OrdersBindingSource_PositionChanged(Nothing, Nothing)
        End If
    End Sub

    Private Sub OrdersBindingSource_PositionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles OrdersBindingSource.PositionChanged
        If optLinqQueries.Checked And Me.OrdersBindingSource.Current IsNot Nothing Then
            'Reload the Order_Details grid
            LoadOrder_DetailsGrid()
        End If
    End Sub
#End Region

    Private Sub btnInsert_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnInsert.Click
        Dim strMsg As String = "INSERT operations with autoincrement primary keys no longer work reliably with Beta 1." + vbCrLf + vbCrLf + _
        "According to the LINQ to SQL team, data binding should be better in Beta 2. This application will be updated when Beta 2 releases."
        MsgBox(strMsg, MsgBoxStyle.Exclamation, "Beta 1 INSERT with Autoincrement Problem")
        Exit Sub

        'Insert Order and Order_Details records for selected Customer ID with random foreign key values
        'Numeric UpDowns control the number of Orders and Order_Details per order
        Application.DoEvents()
        sbLog.Length = 0
        Dim timInsert As New Stopwatch
        timInsert.Start()
        For intOrders As Integer = 1 To CInt(nudAddOrders.Value)
            Dim objOrder As Order_ = CType(OrdersBindingSource.AddNew(), Order_)
            OrdersBindingSource.EndEdit()
            ProcessChanges()
            Dim strProdIDs As String = " "
            strProductIDs = ""
            'The following returned the new OrderID in the March 2007 CTP
            intOrderID = objOrder.OrderID
            'New autoincrement value was lost in Beta 1 (hopefully it will come back)
            For intDetails As Integer = 1 To CInt(nudDetailsPerOrder.Value)
                Dim objDetail As Order_Detail = CType(Order_DetailsBindingSource.AddNew(), Order_Detail)
                strProductIDs += objDetail.ProductID.ToString + "; "
                Order_DetailsBindingSource.EndEdit()
                ProcessChanges()
            Next intDetails
            Application.DoEvents()
        Next intOrders
        OrdersBindingSource.ResetBindings(False)
        Order_DetailsBindingSource.ResetBindings(False)
        OrdersDataGridView.Refresh()
        Order_DetailsDataGridView.Refresh()

        txtTime.Text = Format(timInsert.ElapsedMilliseconds / 1000, "0.###")
        txtLog.Text = sbLog.ToString
        btnSubmitChanges.Enabled = False
    End Sub

    Private Sub OrdersBindingSource_AddingNew(ByVal sender As Object, ByVal e As System.ComponentModel.AddingNewEventArgs) Handles OrdersBindingSource.AddingNew
        'Add a new Order object to the grid
        e.NewObject = GetNewOrder()
        OrdersBindingSource.EndEdit()
        btnSubmitChanges.Enabled = True
    End Sub

    Function GetNewOrder() As Order_
        'Create a new Order_ object with random values for inserts
        Dim objOrder As New Order_
        With objOrder
            .CustomerID = Me.CustomerIDTextBox.Text
            .EmployeeID = CInt(1 + (Rnd() * 8))
            .OrderDate = Today
            .RequiredDate = DateAdd(DateInterval.Day, 14, Today)
            .ShipVia = CInt(1 + (Rnd() * 2))
            .Freight = CDec(Round(10 + (Rnd() * 50), 2))
            .ShipName = Me.CompanyNameTextBox.Text
            .ShipAddress = Me.AddressTextBox.Text
            .ShipCity = Me.CityTextBox.Text
            .ShipRegion = Me.RegionTextBox.Text
            .ShipPostalCode = Me.PostalCodeTextBox.Text
            .ShipCountry = Me.CountryTextBox.Text
        End With
        strProductIDs = ""
        Return objOrder
    End Function

    Private Sub Order_DetailsBindingSource_AddingNew(ByVal sender As Object, ByVal e As System.ComponentModel.AddingNewEventArgs) Handles Order_DetailsBindingSource.AddingNew
        'Add a new Order_Detail object to the grid
        e.NewObject = GetNewDetail()
        Order_DetailsBindingSource.EndEdit()
        btnSubmitChanges.Enabled = True
    End Sub

    Function GetNewDetail() As Order_Detail
        'Create a new Order_Detail object with random values and 
        Dim objDetail As New Order_Detail
        If strProductIDs Is Nothing Then
            'Temporary, shouldn't be required
            strProductIDs = " "
        End If
        With objDetail
            .OrderID = intOrderID
            .ProductID = CInt(1 + (Rnd() * 76))
            While strProductIDs.Contains(.ProductID.ToString)
                .ProductID = CInt(1 + (Rnd() * 76))
            End While
            .Quantity = CShort(1 + Rnd() * 50)
            .UnitPrice = CDec(Round(1 + Rnd() * 75, 2))
            Select Case .Quantity
                Case Is >= 50
                    .Discount = 0.2
                Case Is >= 25
                    .Discount = 0.15
                Case Is >= 10
                    .Discount = 0.1
                Case Is >= 5
                    .Discount = 0.05
            End Select
        End With
        Return objDetail
    End Function

    Private Sub btnUpdate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnUpdate.Click
        'Update Orders and Order_Details with random values
        sbLog.Length = 0
        Dim timUpdate As New Stopwatch
        timUpdate.Start()
        Dim qryOrders = From ord In dcNwind.Orders _
                        Where ord.CustomerID = Me.CustomerIDTextBox.Text _
                        Select ord
        For Each Order As Order_ In qryOrders
            With Order
                .EmployeeID = 1 + CInt(Rnd() * 8)
                'Records added without values other than CustomerID
                .OrderDate = DateAdd(DateInterval.Day, 1, .OrderDate)
                .RequiredDate = DateAdd(DateInterval.Day, 1, .RequiredDate.Value)
                If .ShippedDate.HasValue Then
                    .RequiredDate = DateAdd(DateInterval.Day, 1, .RequiredDate.Value)
                End If
                .ShipVia = 1 + CInt(Rnd() * 2)
                If .Freight.HasValue Then
                    Select Case CInt(.ShipVia)
                        Case 1
                            .Freight = CDec(Round(1.1 * .Freight.Value, 2))
                        Case 2
                            .Freight = CDec(Round(0.9 * .Freight.Value, 2))
                    End Select
                End If
                'Order_Details (don't change ProductID)
                Dim qryDetails = From dtl In dcNwind.Order_Details _
                                 Where dtl.OrderID = .OrderID _
                                 Select dtl
                For Each Detail As Order_Detail In qryDetails
                    With Detail
                        .Quantity = CShort(1 + Rnd() * 50)
                        .UnitPrice = CDec(Round(1 + Rnd() * 75, 2))
                        Select Case .Quantity
                            Case Is >= 50
                                .Discount = 0.2
                            Case Is >= 25
                                .Discount = 0.15
                            Case Is >= 10
                                .Discount = 0.1
                            Case Is >= 5
                                .Discount = 0.05
                        End Select
                    End With
                Next
            End With
        Next
        Application.DoEvents()
        ProcessChanges()
        OrdersDataGridView.Refresh()
        Order_DetailsDataGridView.Refresh()

        txtTime.Text = Format(timUpdate.ElapsedMilliseconds / 1000, "0.###")
        txtLog.Text = sbLog.ToString
    End Sub

    Private Sub btnDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDelete.Click
        Dim strMsg As String = "Click Yes to delete all Orders with OrderID > 11077 or " + vbCrLf + vbCrLf + _
            "Click No to delete the last " + nudAddOrders.Value.ToString + " orders or " + vbCrLf + vbCrLf + _
            "Click Cancel to abandon deleting orders."
        Dim intDelete As Integer = MsgBox(strMsg, MsgBoxStyle.Question Or MsgBoxStyle.YesNoCancel, "Delete Added Orders")
        sbLog.Length = 0
        Dim timDelete As New Stopwatch
        timDelete.Start()
        If intDelete = MsgBoxResult.Yes Then
            'Remove all added orders
            Dim qryDetail = From objDetail In dcNwind.Order_Details _
                        Where objDetail.OrderID > 11077 _
                        Order By objDetail.OrderID Descending _
                        Select objDetail
            For Each objDetail In qryDetail
                dcNwind.Order_Details.Remove(objDetail)
            Next
            Dim qryOrder = From objOrder In dcNwind.Orders _
                        Where objOrder.OrderID > 11077 _
                        Order By objOrder.OrderID Descending _
                        Select objOrder
            For Each objOrder In qryOrder
                dcNwind.Orders.Remove(objOrder)
            Next
        ElseIf intDelete = MsgBoxResult.No Then
            Dim qryOrder = (From objOrder In dcNwind.Orders _
                        Where objOrder.OrderID > 11077 _
                        Order By objOrder.OrderID Descending _
                        Select objOrder).Take(CInt(nudAddOrders.Value))
            For Each objOrder In qryOrder
                Dim qryDetail = From objDetail In dcNwind.Order_Details _
                        Where objDetail.OrderID = objOrder.OrderID _
                        Order By objDetail.OrderID Descending _
                        Select objDetail
                For Each objDetail In qryDetail
                    dcNwind.Order_Details.Remove(objDetail)
                Next
                dcNwind.Orders.Remove(objOrder)
            Next
        Else
            Exit Sub
        End If
        ProcessChanges()
        'Brute force method to remove deleted orders from DataGridView
        btnReload_Click(Nothing, Nothing)
        txtTime.Text = Format(timDelete.ElapsedMilliseconds / 1000, "0.###")
        txtLog.Text = sbLog.ToString
    End Sub

    Private Sub ProcessChanges()
        'Process changes with concurrency conflict resolution
        Try
            dcNwind.SubmitChanges(ConflictMode.ContinueOnConflict)
        Catch ccExc As ChangeConflictException
            'Enumerate the change conflicts for updates and deletions
            For Each ccObject As ObjectChangeConflict In dcNwind.ChangeConflicts
                For Each ccMember As MemberChangeConflict In ccObject.MemberConflicts
                    Dim strMsg As String = dcNwind.ChangeConflicts.Count.ToString + _
                        " concurrency conflict(s) have been detected in Object '" + _
                        ccObject.Object.ToString + "', "
                    With ccMember
                        strMsg += "Member '" + .Member.Name + "'" + vbCrLf + _
                        "Original Value = " + .OriginalValue.ToString + vbCrLf + _
                        "Current Value = " + .CurrentValue.ToString + vbCrLf + _
                        "Database Value = " + .DatabaseValue.ToString + vbCrLf + vbCrLf + _
                        "Click Yes to force the Refresh method to keep the Current value that has been changed, but update the other values with the database values." + vbCrLf + _
                        "Click No to force the Refresh method to swap the Original value with the values retrieved from the database. No current value is modified." + vbCrLf + _
                        "Click Cancel to force the Refresh method to override all the current values with the values from the database."
                        'Resolve each member conflict
                        Dim intResolve As Integer = MsgBox(strMsg, MsgBoxStyle.Exclamation _
                            Or MsgBoxStyle.YesNoCancel, _
                            "Concurency Conflict Resolution for " + ccObject.Object.ToString)
                        If intResolve = MsgBoxResult.Yes Then
                            .Resolve(RefreshMode.KeepChanges)
                        ElseIf intResolve = MsgBoxResult.No Then
                            .Resolve(RefreshMode.KeepCurrentValues)
                        Else
                            .Resolve(RefreshMode.OverwriteCurrentValues)
                        End If
                    End With
                Next
            Next
            'No exception should occur at this point
            dcNwind.SubmitChanges(ConflictMode.FailOnFirstConflict)
        Catch exc As Exception
            MsgBox(exc.Message + vbCrLf + exc.StackTrace, MsgBoxStyle.Exclamation, "Non-Concurrency Exception Submitting Changes")
        End Try
    End Sub


    '*********************************
    ' Handle User DataGridView Changes
    '*********************************

    Private Sub OrdersDataGridView_UserAddedRow(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewRowEventArgs) Handles OrdersDataGridView.UserAddedRow
        OrdersDataGridView.EndEdit()
        btnSubmitChanges.Enabled = True
    End Sub

    Private Sub OrdersDataGridView_UserDeletedRow(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewRowEventArgs) Handles OrdersDataGridView.UserDeletedRow
        OrdersDataGridView.EndEdit()
        btnSubmitChanges.Enabled = True
    End Sub

    Private Sub Order_DetailsDataGridView_DataError(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewDataErrorEventArgs) Handles Order_DetailsDataGridView.DataError
        Dim strMsg As String = "DataGridView data binding no longer works reliably with Beta 1." + vbCrLf + vbCrLf + _
        "According to the LINQ to SQL team, data binding should be better in Beta 2. This application will be updated when Beta 2 releases."
        MsgBox(strMsg, MsgBoxStyle.Exclamation, "Beta 1 DataBinding Problems")
    End Sub

    Private Sub Order_DetailsDataGridView_UserAddedRow(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewRowEventArgs) Handles Order_DetailsDataGridView.UserAddedRow
        Order_DetailsDataGridView.EndEdit()
        btnSubmitChanges.Enabled = True
    End Sub

    Private Sub Order_DetailsDataGridView_UserDeletedRow(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewRowEventArgs) Handles Order_DetailsDataGridView.UserDeletedRow
        Order_DetailsDataGridView.EndEdit()
        btnSubmitChanges.Enabled = True
    End Sub

    Private Sub OrdersDataGridView_UserDeletingRow(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewRowCancelEventArgs) Handles OrdersDataGridView.UserDeletingRow
        'Deletion tracking doesn't seem to work
        Dim intOrderID As Integer = CInt(e.Row.Cells(0).Value)
        Dim strMsg As String = "Click Yes to delete OrderID " + intOrderID.ToString + _
        " or No to retain the order."
        If MsgBox(strMsg, MsgBoxStyle.Question Or MsgBoxStyle.YesNo, "Preparing to Delete OrderID " + intOrderID.ToString) = MsgBoxResult.Yes Then
            'Don't rely on cascading deletion, delete Order_Details first
            Dim qryDetails = From dtl In dcNwind.Order_Details _
                           Select dtl Where dtl.OrderID = intOrderID
            For Each dtl As Order_Detail In qryDetails
                dcNwind.Order_Details.Remove(dtl)
            Next

            'Delete the order
            Dim qryOrder = From ord In dcNwind.Orders _
                           Select ord Where ord.OrderID = intOrderID
            For Each ord As Order_ In qryOrder
                dcNwind.Orders.Remove(ord)
            Next
            ProcessChanges()
        End If
    End Sub

    Private Sub btnSubmitChanges_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        ProcessChanges()
        btnSubmitChanges.Enabled = False
    End Sub

#Region "Customers Move Buttons"
    Private Sub btnMoveFirst_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMoveFirst.Click
        blnIsLoaded = False
        CustomerBindingSource.MoveFirst()
        Application.DoEvents()
        blnIsLoaded = True
    End Sub

    Private Sub btnMovePrevious_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMovePrevious.Click
        If Not CustomerBindingSource.Position = 0 Then
            blnIsLoaded = False
            CustomerBindingSource.MovePrevious()
            Application.DoEvents()
            blnIsLoaded = True
        End If
    End Sub

    Private Sub btnMoveNext_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMoveNext.Click
        If CustomerBindingSource.Position < CustomerBindingSource.Count - 1 Then
            blnIsLoaded = False
            CustomerBindingSource.MoveNext()
            Application.DoEvents()
            blnIsLoaded = True
        End If
    End Sub

    Private Sub btnMoveLast_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMoveLast.Click
        blnIsLoaded = False
        CustomerBindingSource.MoveLast()
        Application.DoEvents()
        blnIsLoaded = True
    End Sub
#End Region

    Private Sub ContactNameTextBox_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ContactNameTextBox.TextChanged
        If blnIsLoaded Then
            btnSubmitChanges.Enabled = True
        End If
    End Sub

    Private Sub btnSubmitChanges_Click1(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnSubmitChanges.Click
        ProcessChanges()
        btnSubmitChanges.Enabled = False
    End Sub
End Class
